@echo off
setlocal enabledelayedexpansion

call :setESC

:: Check if we're running with administrator privileges
net session >nul 2>&1
if %errorlevel% neq 0 (
    :: We're not running with administrator privileges, try to restart ourselves with administrator privileges
    echo Restarting with administrator privileges...
    powershell -Command "Start-Process '%~0' -Verb RunAs"
    exit /b
)

:: Ask the user for the number of GPUs
:input
set "devices="
set /p devices="Enter the number of NVIDIA graphic cards in your computer: "

:: Validate the input
set "valid=true"
for /F "delims=0123456789" %%G in ("!devices!") do set "valid=false"

if "%valid%"=="false" (
    echo Invalid input. Please enter a valid number.
    echo:

    goto input
)

echo This extension will be installed for %devices% GPUs...
set /a "devices-=1"

:: File extensions for which the context menu should appear
set "extensions=bat exe ps1"

:: Generate the run_with_gpu.bat script
> %~dp0run_with_gpu.bat (
    echo @echo off
    echo setlocal
    echo set "original_cuda_visible_devices=%%CUDA_VISIBLE_DEVICES%%"
    echo if "%%~1"=="all" goto :all
    echo set "CUDA_VISIBLE_DEVICES=%%~1"
    echo goto :start
    echo :all
    echo set "CUDA_VISIBLE_DEVICES="
    echo :start
    echo if "%%~x2"==".ps1" ^(
    echo     powershell -File "%%~2"
    echo ^) else ^(
    echo     start "" "%%~2"
    echo ^)
    echo if defined original_cuda_visible_devices ^(
    echo     set "CUDA_VISIBLE_DEVICES=%%original_cuda_visible_devices%%"
    echo     goto :end
    echo ^)
    echo set "CUDA_VISIBLE_DEVICES="
    echo :end
    echo endlocal

)

set "extensionlist="

:: For each extension
for %%e in (%extensions%) do (

    if defined extensionlist (
        set "extensionlist=!extensionlist!, *.%%e"
    ) else (
        set "extensionlist=*.%%e"
    )

    :: Create the cascading menu structure
    reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPUall" /f /ve >NUL
    reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPUall" /f /v "MUIVerb" /t REG_EXPAND_SZ /d "All" >NUL
    reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPUall\Command" /f /ve /d "\"%~dp0run_with_gpu.bat\" all \"%%1\"" >NUL
    reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPUall" /f /v "CommandFlags" /t REG_DWORD /d 64 >NUL

    :: collect all subcommands
    set "subcommands=RunWithGPU.GPUall"

    :: Add the context menu entries for each GPU
    for /l %%i in (0, 1, %devices%) do (
        set "subcommands=!subcommands!;RunWithGPU.GPU%%i"
        reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPU%%i" /f /ve >NUL
        reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPU%%i" /f /v "MUIVerb" /t REG_EXPAND_SZ /d "GPU %%i" >NUL
        reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPU%%i\Command" /f /ve /d "\"%~dp0run_with_gpu.bat\" %%i \"%%1\"" >NUL
    )

    :: Link the cascading menu to the file extension
    reg add "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%e\Shell\RunWithGPU" /f /v "MUIVerb" /t REG_EXPAND_SZ /d "Run with GPU..." >NUL
    reg add "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%e\Shell\RunWithGPU" /f /v "SubCommands" /t REG_SZ /d "!subcommands!" >NUL
    reg add "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%e\Shell\RunWithGPU" /f /v "ExtendedSubCommandsKey" /t REG_SZ /d "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell" >NUL

    :: Set the position of the context menu entry
    reg add "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%e\Shell\RunWithGPU" /f /v "Position" /t REG_SZ /d "Bottom" >NUL

    :: Set the icon for the context menu entry
    reg add "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%e\Shell\RunWithGPU" /f /v "Icon" /t REG_SZ /d "imageres.dll,-149" >NUL
)


:: Generate the uninstaller.bat script
set /a "devices+=1"
> %~dp0uninstall.bat (
    echo @echo off
    echo setlocal enabledelayedexpansion
    echo.
    echo :: Check if we're running with administrator privileges
    echo net session ^>nul 2^>^&1
    echo if %%errorlevel%% neq 0 ^(
    echo     :: We're not running with administrator privileges, try to restart ourselves with administrator privileges
    echo     echo Restarting with administrator privileges...
    echo     powershell -Command "Start-Process '%%~0' -Verb RunAs"
    echo     exit /b
    echo ^)
    echo.
    echo :: File extensions for which the context menu should be removed
    echo set "extensions=!extensions!"
    echo.
    echo :: Delete the cascading menu structure
    echo reg delete "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPUall" /f ^>NUL
    echo.
    echo :: Delete the context menu entries for each GPU
    echo for /l %%%%i in ^(0, 1, !devices!^) do ^(
    echo     reg delete "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\CommandStore\shell\RunWithGPU.GPU%%%%i" /f ^>NUL
    echo ^)
    echo :: For each extension
    echo for %%%%e in ^(%%extensions%%^) do ^(
    echo     :: Remove the context menu entry from HKCU\SystemFileAssociations
    echo     reg delete "HKEY_CLASSES_ROOT\SystemFileAssociations\.%%%%e\Shell\RunWithGPU" /f ^>NUL
    echo ^)
    echo.
    echo echo:
    echo echo Uninstallation completed successfully^^!
    echo echo The context menu entries have been removed for the following filetypes: !extensionlist!
    echo echo:
    echo pause
    echo.
    echo echo Removing run_with_gpu.bat
    echo del "%%~dp0run_with_gpu.bat"
    echo.
    echo echo Removing uninstall.bat
    echo del "%%~f0"
    echo.
    echo endlocal
)

set "URL=donate.48design.com"
:: Display results
echo:
echo Installation completed successfully^^!
echo You can now right-click on the following filetypes and select the GPU to use: !extensionlist!
echo:
echo %ESC%[31mWARNING: Please do not move the files or folder where installer.bat is located.%ESC%[0m
echo If you no longer need the context menu, run uninstaller.bat to uninstall.
echo:

echo %ESC%[93mIt worked^^? Great^^!%ESC%[0m
echo Say thank you with a small donation: %ESC%[92m %URL% %ESC%[0m
echo.
echo Would you like to open %URL%? (Y/N, default is Y)
set /p choice=

if /i "!choice!"=="N" (
    exit
) else (
    start "" "https://%URL%"
    exit
)
echo.

endlocal
pause

:: coloring workaround
:setESC
for /F %%a in ('"prompt $E$S & echo on & for %%b in (1) do rem"') do set "ESC=%%a"
exit /B